part of 'non_bookaable_spaces_bloc.dart';

sealed class NonBookableSpacesState extends Equatable {
  const NonBookableSpacesState();

  @override
  List<Object> get props => [];
}

final class NonBookableSpacesInitial extends NonBookableSpacesState {}

class NonBookableSpacesLoading extends NonBookableSpacesState {
  final PaginatedDataModel<BookableSpacemodel>? lastNonBookableSpaces;
  const NonBookableSpacesLoading({
    this.lastNonBookableSpaces,
  });
}

class NonBookableSpacesLoaded extends NonBookableSpacesState {
  final PaginatedDataModel<BookableSpacemodel> nonBookableSpaces;
  final List<BookableSpacemodel> selectedBookableSpaces;
  const NonBookableSpacesLoaded({
    required this.nonBookableSpaces,
    this.selectedBookableSpaces = const [],
  });
}

class NonBookableSpacesError extends NonBookableSpacesState {
  final String error;
  const NonBookableSpacesError(this.error);
}

class AddNonBookableSpaceIntoBookableState extends NonBookableSpacesState {}

class RemoveBookableSpaceIntoNonBookableState extends NonBookableSpacesState {}

class ValidSaveButtonState extends NonBookableSpacesState {}

class UnValidSaveButtonState extends NonBookableSpacesState {}
